@@ -232,6 +232,9 @@ grl_data_set_string (GrlData *data,
 {
   GValue value = { 0 };
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   if (strvalue) {
     g_value_init (&value, G_TYPE_STRING);
     g_value_set_string (&value, strvalue);
@@ -257,7 +260,12 @@ grl_data_set_string (GrlData *data,
 const gchar *
 grl_data_get_string (GrlData *data, GrlKeyID key)
 {
-  const GValue *value = grl_data_get (data, key);
+  const GValue *value;
+
+  g_return_val_if_fail (GRL_IS_DATA (data), NULL);
+  g_return_val_if_fail (key, NULL);
+
+  value = grl_data_get (data, key);
 
   if (!value || !G_VALUE_HOLDS_STRING (value)) {
     return NULL;
@@ -281,6 +289,10 @@ void
 grl_data_set_int (GrlData *data, GrlKeyID key, gint intvalue)
 {
   GValue value = { 0 };
+
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   g_value_init (&value, G_TYPE_INT);
   g_value_set_int (&value, intvalue);
   grl_data_set (data, key, &value);
@@ -302,7 +314,12 @@ grl_data_set_int (GrlData *data, GrlKeyID key, gint intvalue)
 gint
 grl_data_get_int (GrlData *data, GrlKeyID key)
 {
-  const GValue *value = grl_data_get (data, key);
+  const GValue *value;
+
+  g_return_val_if_fail (GRL_IS_DATA (data), 0);
+  g_return_val_if_fail (key, 0);
+
+  value = grl_data_get (data, key);
 
   if (!value || !G_VALUE_HOLDS_INT (value)) {
     return 0;
@@ -326,6 +343,10 @@ void
 grl_data_set_float (GrlData *data, GrlKeyID key, float floatvalue)
 {
   GValue value = { 0 };
+
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   g_value_init (&value, G_TYPE_FLOAT);
   g_value_set_float (&value, floatvalue);
   grl_data_set (data, key, &value);
@@ -347,7 +368,12 @@ grl_data_set_float (GrlData *data, GrlKeyID key, float floatvalue)
 gfloat
 grl_data_get_float (GrlData *data, GrlKeyID key)
 {
-  const GValue *value = grl_data_get (data, key);
+  const GValue *value;
+
+  g_return_val_if_fail (GRL_IS_DATA (data), 0.0);
+  g_return_val_if_fail (key, 0.0);
+
+  value = grl_data_get (data, key);
 
   if (!value || !G_VALUE_HOLDS_FLOAT (value)) {
     return 0;
@@ -371,6 +397,10 @@ void
 grl_data_set_boolean (GrlData *data, GrlKeyID key, gboolean boolvalue)
 {
   GValue value = { 0 };
+
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   g_value_init (&value, G_TYPE_BOOLEAN);
   g_value_set_boolean (&value, boolvalue);
   grl_data_set (data, key, &value);
@@ -379,7 +409,7 @@ grl_data_set_boolean (GrlData *data, GrlKeyID key, gboolean boolvalue)
 /**
  * grl_data_get_boolean:
  * @data: data to inspect
- * @key (type GrlKeyID): key to use
+ * @key: (type GrlKeyID): key to use
  *
  * Returns: the first boolean value associated with @key from @data. If @key has
  * no first value, or value is not a gboolean, or @key is not in the data, then
@@ -390,7 +420,12 @@ grl_data_set_boolean (GrlData *data, GrlKeyID key, gboolean boolvalue)
 gboolean
 grl_data_get_boolean (GrlData *data, GrlKeyID key)
 {
-  const GValue *value = grl_data_get (data, key);
+  const GValue *value;
+
+  g_return_val_if_fail (GRL_IS_DATA (data), FALSE);
+  g_return_val_if_fail (key, FALSE);
+
+  value = grl_data_get (data, key);
 
   if (!value || !G_VALUE_HOLDS_BOOLEAN (value)) {
     return FALSE;
@@ -417,6 +452,9 @@ grl_data_set_binary (GrlData *data, GrlKeyID key, const guint8 *buf, gsize size)
   GValue v = { 0 };
   GByteArray * array;
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   if (!buf || !size) {
     return;
   }
@@ -449,9 +487,13 @@ grl_data_set_binary (GrlData *data, GrlKeyID key, const guint8 *buf, gsize size)
 const guint8 *
 grl_data_get_binary(GrlData *data, GrlKeyID key, gsize *size)
 {
-  g_return_val_if_fail (size, NULL);
+  const GValue *value;
 
-  const GValue *value = grl_data_get (data, key);
+  g_return_val_if_fail (size, NULL);
+  g_return_val_if_fail (GRL_IS_DATA (data), NULL);
+  g_return_val_if_fail (key, NULL);
+
+  value = grl_data_get (data, key);
 
   if (!value || !G_VALUE_HOLDS_BOXED (value)) {
     return NULL;
@@ -480,6 +522,8 @@ grl_data_set_boxed (GrlData *data, GrlKeyID key, gconstpointer boxed)
 {
   GValue value = { 0 };
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
   g_return_if_fail (boxed != NULL);
 
   g_value_init (&value, GRL_METADATA_KEY_GET_TYPE (key));
@@ -506,7 +550,12 @@ grl_data_set_boxed (GrlData *data, GrlKeyID key, gconstpointer boxed)
 gpointer
 grl_data_get_boxed (GrlData *data, GrlKeyID key)
 {
-  const GValue *value = grl_data_get (data, key);
+  const GValue *value;
+
+  g_return_val_if_fail (GRL_IS_DATA (data), NULL);
+  g_return_val_if_fail (key, NULL);
+
+  value = grl_data_get (data, key);
 
   if (!value || !G_VALUE_HOLDS_BOXED (value)) {
     return NULL;
@@ -515,6 +564,60 @@ grl_data_get_boxed (GrlData *data, GrlKeyID key)
   }
 }
 
+/**
+ * grl_data_set_int64:
+ * @data: data to change
+ * @key: (type GrlKeyID): key to change or add
+ * @intvalue: the new value
+ *
+ * Sets the first int64 value associated with @key in @data. If @key already has a
+ * first value old value is replaced by the new one.
+ *
+ * Since: 0.2.12
+ **/
+void
+grl_data_set_int64 (GrlData *data, GrlKeyID key, gint64 intvalue)
+{
+  GValue value = { 0 };
+
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
+  g_value_init (&value, G_TYPE_INT64);
+  g_value_set_int64 (&value, intvalue);
+  grl_data_set (data, key, &value);
+}
+
+/**
+ * grl_data_get_int64:
+ * @data: data to inspect
+ * @key: (type GrlKeyID): key to use
+ *
+ * Returns the first int64 value associated with @key from @data. If @key has no
+ * first value, or value is not a gint, or @key is not in data, then 0 is
+ * returned.
+ *
+ * Returns: int value associated with @key, or 0 in other case.
+ *
+ * Since: 0.2.12
+ **/
+gint64
+grl_data_get_int64 (GrlData *data, GrlKeyID key)
+{
+  const GValue *value;
+
+  g_return_val_if_fail (GRL_IS_DATA (data), 0);
+  g_return_val_if_fail (key, 0);
+
+  value = grl_data_get (data, key);
+
+  if (!value || !G_VALUE_HOLDS_INT64 (value)) {
+    return 0;
+  } else {
+    return g_value_get_int64 (value);
+  }
+}
+
 /**
  * grl_data_remove:
  * @data: data to change
@@ -528,6 +631,9 @@ grl_data_get_boxed (GrlData *data, GrlKeyID key)
 void
 grl_data_remove (GrlData *data, GrlKeyID key)
 {
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   grl_data_remove_nth (data, key, 0);
 }
 
@@ -550,6 +656,7 @@ grl_data_has_key (GrlData *data, GrlKeyID key)
   gboolean found = FALSE;
 
   g_return_val_if_fail (GRL_IS_DATA (data), FALSE);
+  g_return_val_if_fail (key, FALSE);
 
   sample_key = get_sample_key (key);
   if (!sample_key) {
@@ -672,6 +779,9 @@ grl_data_add_string (GrlData *data,
 {
   GrlRelatedKeys *relkeys;
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   if (strvalue) {
     relkeys = grl_related_keys_new ();
     grl_related_keys_set_string (relkeys, key, strvalue);
@@ -696,6 +806,9 @@ grl_data_add_int (GrlData *data,
 {
   GrlRelatedKeys *relkeys;
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   relkeys = grl_related_keys_new ();
   grl_related_keys_set_int (relkeys, key, intvalue);
   grl_data_add_related_keys (data, relkeys);
@@ -718,6 +831,9 @@ grl_data_add_float (GrlData *data,
 {
   GrlRelatedKeys *relkeys;
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   relkeys = grl_related_keys_new ();
   grl_related_keys_set_float (relkeys, key, floatvalue);
   grl_data_add_related_keys (data, relkeys);
@@ -742,6 +858,9 @@ grl_data_add_binary (GrlData *data,
 {
   GrlRelatedKeys *relkeys;
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
   if (!buf || !size) {
     return;
   }
@@ -768,6 +887,8 @@ grl_data_add_boxed (GrlData *data,
 {
   GrlRelatedKeys *relkeys;
 
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
   g_return_if_fail (boxed != NULL);
 
   relkeys = grl_related_keys_new ();
@@ -775,6 +896,31 @@ grl_data_add_boxed (GrlData *data,
   grl_data_add_related_keys (data, relkeys);
 }
 
+/**
+ * grl_data_add_int64:
+ * @data: data to append
+ * @key: (type GrlKeyID): key to append
+ * @intvalue: the new value
+ *
+ * Appends a new int64 value for @key in @data.
+ *
+ * Since: 0.2.12
+ **/
+void
+grl_data_add_int64 (GrlData *data,
+                    GrlKeyID key,
+                    gint64 intvalue)
+{
+  GrlRelatedKeys *relkeys;
+
+  g_return_if_fail (GRL_IS_DATA (data));
+  g_return_if_fail (key);
+
+  relkeys = grl_related_keys_new ();
+  grl_related_keys_set_int64 (relkeys, key, intvalue);
+  grl_data_add_related_keys (data, relkeys);
+}
+
 /**
  * grl_data_length:
  * @data: a data
